; AXE133 Serial OLED using PICAXE-18M2
; 1107/2048
; Emulates basic serial operation of the popular AXE033 module
; CPS, May 2011
; Modified by MRBurnette 2011-12-22 to incorporate features
; Modified by MRBurnette 2011-12-21 to incorporate display line-buffers
; Modified by MRBurnette 2011-12-09 messages 0/1
; Modified by MRBurnette 2011-12-01 to remove non-OLED code

#picaxe 18M2
{
; Supported Commands
; 0-7, 8-15	CGRAM characters
; 16-252	normal ASCII characters, according to selected character map table
; 253, X	display 16 character pre-saved message from EEPROM memory, X can be 0-15
; 254, X	LCD command, X can be 0 to 255 
; 255, X	control outputs C.2, C.1, C.0 (via lower 3 bits of X)
;		So, if using a backlit LCD with the active low transistor driver
;		on output C.2, then 255,%000 switches backlight on and 255,%100 switches off

symbol line_length = 16	; change to 20 for displays with 20 character lines
symbol baud =  N2400_16	; Serial baud rate 2400,N,8,1. Note main program runs at 16MHz
symbol spare0 	= C.0 ; spare input/output
symbol spare1 	= C.1 ; IR pulse input
symbol spare2 	= C.2 ; 1 second pulse input from external clock
symbol backlight 	= C.2 ; optional backlight control for backlit LCDs, active low
symbol RX		= C.5	; serial receive pin
symbol enable 	= C.6	; LCD enable
symbol rs 		= C.7	; LCD RS: rs = low for command mode / rs = high for character mode

;---------------
;SYMBOL table - aliases for byte values
SYMBOL Line		= b0		; values = 0/1 indicating OLED display line
;RESERVED		= b1		; b1-b4 original AXE133Y variables used in firmware
;RESERVED		= b2		; ditto
;RESERVED		= b3		; ditto
;RESERVED		= b4		; ditto
SYMBOL DispFlag 	= B5		; Value controls the Select Case display choices
SYMBOL ISset	= B6		; ISset is a logical value, false on power-up
SYMBOL secs       = B7		; secs / mins / hours = time numeric values
SYMBOL mins       = B8		; ditto
SYMBOL hour       = B9		; ditto
SYMBOL InfraRed   = B10		; IR returned integer: 0 - 9
SYMBOL tempL	= B11		; Low-byte
SYMBOL tempH	= B12		' Hi-byte



;SYMBOL table - aliases for system values
'Symbol Serial = C.1		Removed as 18M2 implementation is 'self-contained'


;16 characters of display and 2 optional control characters
Poke 28, 30 : Poke 29, 47 : Poke 30, 0, 0
Poke 56, 58 : Poke 57, 75 : Poke 58, 0, 0

;------------------

; LCD data pins are on B.0 to B.7 on 18M2

; Store the 16 character user defined messages in EEPROM data memory
; First two messages are optionally used as welcome message

; If using a display with 20 characters you will need to edit 
; the start addresses to be multiples of 20 (currently at 16) 
; and add 4 characters to each message.
; Please remember 4 line displays always use the strange 1-3-2-4 layout.

}

;Stored Messages
{
EEPROM $00, ("M. Ray Burnette ") ; MSG 0
EEPROM $10, ("SofKinetics, Inc") ; MSG 1
EEPROM $20, ("  Clock Setup   ") ; MSG 2
EEPROM $30, (" Use TV Remote  ") ; MSG 3
EEPROM $40, ("Current time:   ") ; MSG 4
EEPROM $50, ("                ") ; MSG 5 = blank line
EEPROM $60, ("                ") ; MSG 6 = RESERVED
EEPROM $70, ("                ") ; MSG 7 = RESERVED
EEPROM $80, ("                ") ; MSG 8 = RESERVED
EEPROM $90, ("                ") ; MSG 9 = RESERVED
EEPROM $A0, ("                ") ; MSG A = RESERVED
EEPROM $B0, ("                ") ; MSG B = RESERVED
EEPROM $C0, ("                ") ; MSG C = RESERVED
EEPROM $D0, ("                ") ; MSG D = RESERVED
EEPROM $E0, ("                ") ; MSG E = RESERVED
EEPROM $F0, ("                ") ; MSG F = RESERVED

}

init:
{
	gosub LCD_init 		; initialise LCD

welcome:
	let b1 = 0			; message 0 on top line
	gosub msg			; do it

	low rs			; command mode
	let pinsB = 192		; move to line 2, instruction 192
	pulsout enable,1  	; pulse the enable pin to send data.
	high rs			; character mode again
	
	let b1 = 1			; message 1 on bottom line
	gosub msg			; do it
		
}
	
Top:
{
	; Send startup message to serial port
	DispFlag = 0 : Gosub Display

	If ISset = 0 Then
		DispFlag = 1 : Gosub Display : PAUSE 8000
		Goto SetClock
	Else
		DispFlag = 4 : GoSub Display
		DispFlag = 5 : GoSub Display
	EndIf

	Pause 4000


	; The clock pulse is 50ms + pulse of 4.5V from an old clock movement
Do
	Do while PINC.0 = 0 : Loop
	
	secs = secs + 1 // 60

	If secs = 0 Then
		mins = mins + 1 // 60
      	If mins = 0 Then
      		hour = hour + 1 // 24
      	EndIf
      EndIf

	Do while PINC.0 = 1 : Loop
	
	; now have about 950mS to do whatever (worst-case, 900mS)
	DispFlag = 2 : GoSub Display
	
Loop
}

Display:
{
	Select Case DispFlag

  	Case 0
  		POKE 30, 254, 1 : GoSub DisplayRAM	; Send "clear screen" to OLED
  		; Clear the buffer Line 0
		POKE 30,0,0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,0,0,0,0,0,0
		; Clear the buffer Line 1
		POKE 58,0,0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,0,0,0,0,0,0
  		
  	Case 1
  		POKE 30, 254, 128 : GoSub DisplayRAM
  		POKE 30,0,0,"C","l","o","c","k"," ","n","o","t"," ","s","e","t",".",".","."
		Line = 0 : gosub DisplayRAM
 
 	Case 2	; Called every second to update time on OLED bottom line
 			;_____RAM MAP________
 			;55556666666666777777
 			;67890123456789012345
 			;sfcc....HH:MM:SS....   s=start, f=finish, c/c=control_bytes
 			;
 		POKE 58,254,192
 		Line = 1 : GoSub DisplayRAM
 		POKE 58,0,0
 		
 		If hour = 0 then
 			; EQ = serout Serial,BAUD,("    00:")
 			POKE 64,"0","0",":"
 		elseif hour < 10 then
 			; EQ = serout Serial,BAUD,("    0",#hour,":")
 			tempL = hour + 48
 			POKE 64,"0",tempL,":"
 		else
 			; EQ = serout Serial,BAUD,("    ",#hour,":")
 			tempL = hour //10 + 48
 			tempH = hour / 10 + 48
 			POKE 64,tempH,tempL,":"
 		endIf
 		
 		If mins = 0 then
 			POKE 67, "0","0",":"
 		elseif mins < 10 then
 			tempL = mins + 48
 			POKE 67, "0",tempL,":"
 		else
 			tempL = mins //10 + 48
 			tempH = mins / 10 + 48
 			POKE 67,tempH,tempL,":"
 		endIf
 		
 		If secs = 0 then
 			POKE 70,"0","0"
 		elseif secs < 10 then
 			tempL = secs + 48
 			POKE 70,"0",tempL
 		else
 			tempL = secs //10 + 48
 			tempH = secs / 10 + 48
 			POKE 70, tempH, tempL
 		endIf
 		
 		GoSub DisplayRAM
 
 	Case 3	; Multiline messages
  			; Line 0 Pos 1 ---> MSG 2
  		POKE 30,254,128
 		Line = 0 : GoSub DisplayRAM
  		b1 = 2 : GoSub msg
  		
  			; Line 1 Pos 1 ---> MSG 3
  		POKE 58,254,192
  		Line = 1 : GoSub DisplayRAM
  		b1 = 3 : GoSub msg
 
 	Case 4	; Line 0 Pos 1 ---> MSG 4
  
  		POKE 30,254,128
  		Line = 0 : GoSub DisplayRAM
  		b1 = 4 : GoSub msg
  
  	Case 5	; Line 1 Pos 1 ---> MSG 5
  
  		POKE 58,254,192
  		Line = 1 : GoSub DisplayRAM
  		b1 = 5 : GoSub msg
  		
  	EndSelect
  
	Return
}

DisplayRAM:
{
	; Line variable must be 0 or 1 for OLED display
	;RAM bank #1/2:
	;2233333333334444444444555555	' RAM BAND 1 = Line Buffer #0
	;8901234567890123456789012345
	;
	;5555666666666677777777778888 ' RAM BANK 2 = Line Buffer #1
	;6789012345678901234567890123
	
	If Line = 0 then
		Peek 28, b3 : Peek 29, b4	; Get Display range "begin" Line #0
		Peek 30, b1				; First Control byte if != 0
	Else
		Peek 56, b3 : Peek 57, b4	; Get Display range "end"   Line #1
		Peek 58, b1				; First Control byte if != 0
	EndIf
	
	If b1 < 253 then			; Nothing special if < 253
		b3 = b3 + 2			; No control codes - jump over 'em
		For b2 = b3 to b4		; FOR loops always execute once!
			Peek b2, b1		; serializer for/next loop
			if b1 = 27 then	; Is character ESC ?
				EXIT		; jump out of loop if ESC
			endif
			let pinsB = b1 	; output the data on port_B
			pulsout enable,1  ; pulse the enable pin to send data
		Next

	elseIf b1 = 254 then		; OLED Display Command
		low rs			; Activate Command Mode
		If Line = 0 then
			Peek 31, b1		; Get OLED command byte for first line
		else
			Peek 59, b1		; Get OLED command byte for second line
		EndIf
		let pinsB = b1		; Output OLED command
		pulsout enable,1
		high rs			; Return to Character Mode

	elseIf b1 = 253 then		; User Command Mode - currently EEPROM messages
		If Line = 0 then
			Peek 31, b1		; Get Message # for first line
		else
			Peek 59, b1		; Get Message # for second line
		EndIf
		
		gosub msg			; Display EEPROM message

	EndIf

	Return
}

SetClock:
{
	hour = 00 : mins = 00 : secs = 00 : Line = 1
	DispFlag = 3 : GoSub Display
	Pause 6000
	DispFlag = 5 : GoSub Display
	
	HOURS:
	; X_:__:__
	GoSub GetIR
	hour = InfraRed * 10
	IF hour > 20 then HOURS
	DispFlag = 2 : GoSub Display

	; _X:__:__
	GoSub GetIR
	hour = hour + InfraRed
	IF hour > 24 then HOURS
	DispFlag = 2 : GoSub Display

	MINUTES:
	; __:X_:__
	GoSub GetIR
	mins = InfraRed * 10
	If mins > 50 then MINUTES
	DispFlag = 2 : GoSub Display

	; __:_X:__
	GoSub GetIR
	mins = mins + InfraRed
	If mins > 60 then MINUTES
	DispFlag = 2 : GoSub Display

	SECONDS:
	; __:__:X_
	GoSub GetIR
	secs = InfraRed * 10
	If secs > 50 then SECONDS
	GoSub Display

	; __:__:_X
	GoSub GetIR
	secs = secs + InfraRed
	If secs > 60 then SECONDS
	GoSub Display

	ISset = 1
	GoTo Top
}

GetIR:
{
	IRIN C.1, InfraRed
	
	Pause 1000

	If InfraRed < 9 then
		INC InfraRed
	else
		InfraRed = 0
	EndIf

	Return
}

LCD_init:	; power on OLED initialisation sub routine
{
	let dirsC = %11000000	; PortC 6,7 outputs  MODIFIED C.0, C.1, C.2 Inputs
	let dirsB = %11111111	; PortB all outputs
	; Winstar OLED Module Initialisation
	; according to WS0010 datasheet (8 bit mode)

	pause 500 			; Power stabilistation = 500ms

	; Function set - select only one of these 4 character table modes
	;let pinsB = %00111000 	; 8 bit, 2 line, 5x8 , English_Japanese table
	let pinsB = %00111001 	; 8 bit, 2 line, 5x8 , Western_European table1
	;let pinsB = %00111010 	; 8 bit, 2 line, 5x8 , English_Russian  table
	;let pinsB = %00111011 	; 8 bit, 2 line, 5x8 , Western_European table2
	
	pulsout enable,1  	; 
		
	let pinsB = %00001100	; Display on, no cursor, no blink
	pulsout enable,1 	

	let pinsB = %00000001 	; Display Clear
	pulsout enable,1
	pause 7			; Allow 6.2ms to clear display

	setfreq m16			; now change to 16Mhz

	let pinsB = %00000010 	; Return Home
	pulsout enable,1

	let pinsB = %00000110 	; Entry Mode, ID=1, SH=0
	pulsout enable, 1

	high rs			; Leave in character mode
	return

}

msg:	; display message from EEPROM sub routine
{
	; message number 0-15 must be in b1 when called
	; uses (alters) b1, b2, b3, b4
	let b2 = b1 & %00001111 * line_length
						; EEPROM start address is 0 to 15 multiplied by 16
	let b3 = b2 + line_length - 1 ; end address is start address + (line_length - 1)
	for b4 = b2 to b3			; for 16 times
		read b4,b1			; read next character from EEPROM data memory into b1
		let pinsB = b1 		; output the data
		pulsout enable,1  	; pulse the enable pin to send data.
	next b4				; next loop
	return
}

END

; REMARKS ON DISPLAY RAM STRUCTURE
{
#REM
;RAM Buffer #1 starts at offset 28
;RAM Buffer #2 starts at offset 56
;RAM Buffer #3 starts at offset 84 (not currently used)
;
;RAM display buffer structure:
;
RAM bank #1:
2233333333334444444444555555
8901234567890123456789012345
FFCCD              |XXXXXXXX  RAM Not Used (Reserved)
IIOOI              |
RNNNG              |--->      OLED off-screen 17-20= RAM 48-51
SATTI              |
TLRRT              |
..OO#        1     |
##LL1234567890123456          Visible 16 characters
 
RAM bank #2:
5555666666666677777777778888
6789012345678901234567890123
FFCCD              |XXXXXXXX  RAM Not Used (Reserved)
IIOOI              |
RNNNG              |--->      OLED off-screen 17-20= RAM 76-79
SATTI              |
TLRRT              |
..OO#        1     |
##LL1234567890123456          Visible 16 characters

General notes:
bank #1 is for OLED line #1
bank #2 is for OLED line #2
byte 28/56 is the first character to display ... Normally= 30/58  (2 ctrl + 16)
byte 29/57 is the last character to display  ... Normally= 47/75
byte 30+31 is control sequence line #1       ... 0 = null = not sent
byte 58+59 is control sequence line #2       ... 0 = null = not sent
byte 32-47 / 60-75 are character positions L1:1-16 and L2: 1-16

#endrem
}